! Dean Townsley 2010
!
! init block for surface helium detonation on WD

subroutine Simulation_initBlock(blockID)
  
  use Simulation_data
  use Grid_interface, ONLY : Grid_getBlkIndexLimits, &
    Grid_getBlkBoundBox, Grid_getDeltas, Grid_putPointData, &
    Grid_getCellCoords, Grid_getBlkCornerID, Grid_getMinCellSize
  use Eos_interface, ONLY : Eos

  implicit none
#include "Flash.h"
#include "constants.h"
#include "Eos.h"

  
  integer, intent(in) :: blockID

  integer :: i, j, k, n

  integer, dimension(2,MDIM) :: blkLimits, blkLimitsGC
  integer, dimension(MDIM) :: cornerID, stride, cell

  real :: dens, temp, he, enuc

  real, dimension(EOS_NUM) :: eosData
  real, dimension(NSPECIES) :: abunData

  real, allocatable, dimension(:) :: iCoords, jCoords, kCoords
  integer :: isizeGC, jsizeGC, ksizeGC
  real :: min_dx, xp, ign_dist

  integer :: gi


!==============================================================================

  ! get essential info about this block
  call Grid_getBlkIndexLimits(blockID,blkLimits,blkLimitsGC)

  call Grid_getBlkCornerID(BlockID,cornerID,stride)

  isizeGC = blkLimitsGC(HIGH,IAXIS)
  allocate(iCoords(isizeGC))
  jsizeGC = blkLimitsGC(HIGH,JAXIS)
  allocate(jCoords(jsizeGC))
  ksizeGC = blkLimitsGC(HIGH,KAXIS)
  allocate(kCoords(ksizeGC))
  call Grid_getCellCoords(IAXIS,blockID,CENTER,.true.,iCoords,isizeGC)
  call Grid_getCellCoords(JAXIS,blockID,CENTER,.true.,jCoords,jsizeGC)
  call Grid_getCellCoords(KAXIS,blockID,CENTER,.true.,kCoords,ksizeGC)

  call Grid_getMinCellSize(min_dx)

  ! loop over all zones and init
  do k = blkLimits(LOW,KAXIS), blkLimits(HIGH,KAXIS)
     do j = blkLimits(LOW,JAXIS), blkLimits(HIGH,JAXIS)
        do i = blkLimits(LOW,IAXIS), blkLimits(HIGH,IAXIS)

           dens=0.0
           temp=0.0
           he  =0.0
           do n = 1,stride(IAXIS)
              ! global iaxis index for this (sub) point
              gi = cornerID(IAXIS)+stride(IAXIS)*(i-NGUARD-1)+n-1
              ! x position of this (sub-)point
              xp = (gi-0.5)*min_dx
              ! away from flame interface (most of domain)
              ! just use the overal HSE
              dens = dens+sim_dens_i(gi)
              temp = temp+sim_temp_i(gi)
              he   = he+  sim_he_i(gi)
           enddo
           dens = dens/stride(IAXIS)
           temp = temp/stride(IAXIS)
           he = he/stride(IAXIS)

           ign_dist = (iCoords(i)-sim_ign_x)**2
           if (NDIM>=2) ign_dist = ign_dist + (jCoords(j)-sim_ign_y)**2
           if (NDIM==3) ign_dist = ign_dist + (kCoords(k)-sim_ign_z)**2
           ign_dist = sqrt(ign_dist)

           enuc = 0.0
           if (ign_dist <= sim_ign_r .and. iCoords(i) >= sim_x0) then
             temp = max(temp, sim_ign_temp_center - &
                            (sim_ign_temp_center-sim_ign_temp_edge)*ign_dist/sim_ign_r )
             enuc = 1.1*sim_refNogenEnucThresh
           endif

           eosData(EOS_DENS) = dens
           eosData(EOS_TEMP) = temp
           abunData(:) = 0.0
           abunData(HE4_SPEC-SPECIES_BEGIN+1) = he
           if (iCoords(i) < sim_x0) then
             abunData(MG24_SPEC-SPECIES_BEGIN+1) = 1.0-he
           elseif (iCoords(i) < sim_cz_base_position) then
             abunData(C12_SPEC-SPECIES_BEGIN+1) = sim_x_c12_under
             abunData(O16_SPEC-SPECIES_BEGIN+1) = sim_x_o16_under
           else
             abunData(C12_SPEC-SPECIES_BEGIN+1) = sim_x_c12_cz
             abunData(O16_SPEC-SPECIES_BEGIN+1) = sim_x_o16_cz
           endif
           call Eos(MODE_DENS_TEMP,1,eosData,abunData)
           
           cell(IAXIS) = i
           cell(JAXIS) = j
           cell(KAXIS) = k
           call Grid_putPointData(blockId, CENTER, DENS_VAR, EXTERIOR, cell, dens)
           call Grid_putPointData(blockId, CENTER, TEMP_VAR, EXTERIOR, cell, temp)
           call Grid_putPointData(blockId, CENTER, HE4_SPEC, EXTERIOR, cell, he)
           call Grid_putPointData(blockId, CENTER, MG24_SPEC, EXTERIOR, cell, abunData(MG24_SPEC-SPECIES_BEGIN+1) )
           call Grid_putPointData(blockId, CENTER, C12_SPEC, EXTERIOR, cell, abunData(C12_SPEC-SPECIES_BEGIN+1) )
           call Grid_putPointData(blockId, CENTER, O16_SPEC, EXTERIOR, cell, abunData(O16_SPEC-SPECIES_BEGIN+1) )

           call Grid_putPointData(blockId, CENTER, ENUC_VAR, EXTERIOR, cell, enuc)

           call Grid_putPointData(blockId, CENTER, VELX_VAR, EXTERIOR, cell, 0.0)
           call Grid_putPointData(blockId, CENTER, VELY_VAR, EXTERIOR, cell, 0.0)
           call Grid_putPointData(blockId, CENTER, VELZ_VAR, EXTERIOR, cell, 0.0)

           call Grid_putPointData(blockId, CENTER, ENER_VAR, EXTERIOR, cell, eosData(EOS_EINT))
           call Grid_putPointData(blockId, CENTER, EINT_VAR, EXTERIOR, cell, eosData(EOS_EINT))
           call Grid_putPointData(blockId, CENTER, PRES_VAR, EXTERIOR, cell, eosData(EOS_PRES))
           call Grid_putPointData(blockId, CENTER, GAMC_VAR, EXTERIOR, cell, eosData(EOS_GAMC))
           call Grid_putPointData(blockId, CENTER, GAME_VAR, EXTERIOR, cell, &
                                       eosData(EOS_PRES)/(dens*eosData(EOS_EINT))+1.0)
        enddo

     enddo
  enddo
  
  deallocate(iCoords)
  deallocate(jCoords)
  deallocate(kCoords)

  return
  
end subroutine Simulation_initBlock
